pro milca_apply_weights, map_file_list, dir_w, tszmap
;+
; NAME:
;
;  milca_apply_weights
;
; PURPOSE:
;
;  Apply MILCA weights from Planck SZ relase PR2-2015
;
; INPUTS:
;
; - map_file_list : string array containing the path and name of the files for the
;   frequency maps to which the weights will be applied. Assuming fits files
; - dir_w: string indicating the name of the directory where the MILCA
;   weights are stored
;
; OUTPUTS:
;
;  tszmap : final combined map
;
; KEYWORDS:
;
;
; EXAMPLE:
;
;  frstr = ['100','143','217','353-psb','545','857']
;  dir_in_maps = '/rdata/macias/Planck/PR3-2018/'
;  map_file_list = dir_in_maps + 'HFI_SkyMap_'+frstr+'_2048_R3.00_full.fits'
;  dir_w = '/data/Workspace/macias/Documents/PLANCK/SZ/DX11d/MILCA/CSZ_DX11d/'
;  milca_apply_weights, map_file_list, dir_w, tszmap
;
; MODIFICATION HISTORY:
;
;  First version updated from MILCA code (G. Hurier, J.F. Macias-Perez,
;  S. Hildebrand, B. Comis 2010) by J.F. Macias-Perez, July 2018
;
;-


  Kcmb2Mjy = [1.0,1.0,1.0,1.0,57.9872,2.2692]
  beam = [9.66,7.27,5.01,4.86,4.84,4.63] 
  ang = [5.0,7.50,10.0000,13.4132,18.7716,25.2406,33.2659,43.5919,57.5805,78.0786,112.465,190.082,600.0,1500.0,3600.0,5.0]


  farr = ['100','143','217','353','545','857']
  nf = n_elements(farr)

  nside = 2048.0
  npix = 12.0 * nside^2.0

  ;;  define final tsSZ map
  tszmap =  dblarr(npix)


  initmask = 1
  for index = 2, 12 do begin ; loop in resolution
    res = ang[index:index + 1]
    print, "Working at resolution :", res

    ; Select frequency maps to be used for the actual resolution
    if index eq 0 then sel = [3, 4, 5]
    if index eq 1 then sel = [1, 2, 3, 4, 5]
    if index ge 2 and index le 6 then sel = [0, 1, 2, 3, 4, 5]
    if index gt 6 then sel = [0,1,2,3,4,5]
    ; Compute min and max resolution of the filter
    reso1 = sqrt(res[0]^2.0-beam[sel]^2)
    reso2 = sqrt(res[1]^2.0-beam[sel]^2)

    if initmask eq 1 then begin      
       mtmp = fltarr(npix)+1.0

    ; Read all data and Check possible bad pixels
       for i = 0, n_elements(sel)-1 do BEGIN
          map_obj =  map_file_list[sel[i]]
          read_fits_map, map_obj, map10
          map10 = map10[ * , 0]
          ud_grade, map10, map10, order_in = 'nested', order_out = 'ring'
          pm = where(map10 lt -1.0d20, npm)
          if npm ne 0 then mtmp[pm] = 0.0d0
       ENDFOR
       initmask = 0
    endif  

    ; Loop over all frequencies maps used for the actual resolution 
    for i = 0, n_elements(sel)-1 do begin
        map_obj=map_file_list[sel[i]]
        read_fits_map, map_obj, map10
        map10 = map10[ * , 0] ; assuming multiple maps in the same extension of the fits file
        ud_grade, map10, map10, order_in = 'nested', order_out = 'ring' ; assume maps in nested order
        map10 = map10/Kcmb2Mjy[i] ; convert into KCMB units
        map10 *= mtmp
        pm = where(map10 lt -1.0d20,npm)
        if npm ne 0 then map10[pm] = 0.0d0
        if npm ne 0 then mtmp[pm] = 0.0d0

        ; For all possible resolutions in the decomposition
        ; compute smoothing maps
        ; read and filter the data
        ismoothing,map10*mtmp,map10s1,fwhm=reso1[i],/ring
        if res[1] ne 5.0 then ismoothing,map10*mtmp,map10s2,fwhm=reso2[i],/silent,/ring else map10s2 = map10*0.0
        mapfilt = (map10s1-map10s2)-mean(map10s1-map10s2)
        IF keyword_set(savesm) THEN BEGIN 
          obj_name = farr[sel[i]] + 'I_' + strtrim(long(res[0]), 2) + '_' + strtrim(long(res[1]), 2)  + '.save'
          save, filename = obj_name, mapfilt
        ENDIF
        ; read weights from fits files
        wstruc =  mrdfits(dir_w + 'MILCA_Csz_' +farr[sel[i]] + 'GHz.fits',1)
        istruct =  index -2 ; this is because we start computing the data for 10 arcmin resolution
        ; apply weights to the filtered map
        tszmap += mapfilt * wstruc.(istruct)
    endfor

 endfor ;end loop over all resolutions


end
